############################################################################################################
# INPUT: 1 GCT file, 1 CLS file
# OUTPUT: 1 GCT file with replicates combined into one mean class value, and a new corresponding CLS file
############################################################################################################

####### ---------------------------- #######
#######       READ GCT FILE          #######
####### ---------------------------- #######
read.gct <- function(datafile) {
	x = read.table(datafile, as.is=T, skip=2, header=T, sep="\t", quote="\"", check.names=F, fill=TRUE, comment.char="")
	rownames(x) = as.character(x[,1])
	return(x)
}

####### ---------------------------- #######
#######       READ CLS FILE          #######
####### ---------------------------- #######
read.cls <- function(clsFile="") {
	if(clsFile=="") {
		clsFile = choose.files(multi=F, filter=c("CLS file","*.cls"))
		if(length(clsFile)==0)
			return()
	}	
	x = list()
	classes	= 1+c(-1,-1, scan(file=clsFile, what=integer(0), skip=2, quiet=T))
	numClasses = max(classes)
	classNames = scan(file=clsFile, sep=" ", what=character(0), nmax=numClasses+1, skip=1, quiet=T)[-1]
	x$classNames = classNames
	x$membership = classes	
	return(x)
}

####### ---------------------------- #######
#######       READ CLM FILE          #######
####### ---------------------------- #######
read.clm <- function(clmFile) {
	x = read.table(clmFile, sep="\t", header=F, quote="\"", check.names=F, comment.char="")
	rownames(x) = x[,2]
	colnames(x) = c("CEL File", "Sample Name", "Population")
	return(x)
}

####### ---------------------------- #######
#######      WRITE GCT FILE          #######
####### ---------------------------- #######
write.gct <- function(dataset, filename) {
	EOL = "\n"
	h1 = c("#1.2", EOL)
	h2 = c(nrow(dataset), "\t", ncol(dataset)-2, EOL)
	h3 = c(paste(colnames(dataset), collapse="\t"), EOL)
	cat(h1, file=filename, append=F, sep="")
	cat(h2, file=filename, append=T, sep="")
	cat(h3, file=filename, append=T, sep="")
	write.table(dataset, file=filename, sep="\t", append=TRUE, quote=FALSE, col.names=FALSE, row.names=FALSE, eol=EOL)
}

####### ---------------------------- #######
#######      WRITE CLS FILE          #######
####### ---------------------------- #######
write.cls <- function(dat, clm, filename) {
	clm[,3] = toupper(clm[,3])
	classes = unique(as.character(clm[,3]))
	cls = array(0, ncol(dat)-2)
	for(i in 1:length(classes))
		cls[which(colnames(dat)[-1:-2] %in% clm[clm[,3]==classes[i],2])] = i-1
	write(c(length(cls), max(cls)+1, 1), filename, sep=" ")
	write(c("#", classes), filename, ncolumns=length(classes)+1, append=T)
	write(cls, filename, ncolumns=length(cls), append=T)
}

####### ---------------------------- #######
#######        MAIN FUNCTION         #######
####### ---------------------------- #######
classMean <- function(gctFile, clsFile, log.transform=1, precision=5) {
	log.xform  = as.integer(log.transform) == 1
	precision  = as.integer(precision) 
	dataset    = read.gct(gctFile)
	classes    = read.cls(clsFile)
	#stopifnot(length(classes$membership) != ncol(dataset)-2)
	numChips   = length(classes$membership)
	numClasses = length(classes$classNames)
	samples    = colnames(dataset)[-1:-2]
	dataMean   = cbind(dataset[,1:2], matrix(,nrow(dataset),numClasses))
	colnames(dataMean)[-1:-2] = classes$classNames

	if(log.xform==T) {
		cat("Log-transforming data...")
		dataset[,-1:-2] = log2(dataset[,-1:-2])
	}
	for(i in 1:numClasses)
		dataMean[,2+i] = rowMeans(as.matrix(dataset[,classes$membership==i]))
	if(log.xform==T)
		dataMean[,-1:-2] = 2^dataMean[,-1:-2]
	if(precision<9)
		dataMean[,-1:-2] = round(dataMean[,-1:-2], digits=precision)

	gctFile = basename(gctFile)
	clmData = cbind(array(,numClasses), classes$classNames, classes$classNames)
	gctFile = paste(substring(gctFile, 1, nchar(gctFile)-4), "- class means.gct")
	clsFile = paste(substring(gctFile, 1, nchar(gctFile)-4), ".cls", sep="")
	cat(clsFile,"\n")
	cat(gctFile,"\n")
	write.gct(dataMean, gctFile)
	write.cls(dataMean, clmData, clsFile)
}

